@@ -36,6 +36,7 @@
 #include <sys/select.h>
 #include <sys/uio.h>
 #include <arpa/inet.h>
+#include <signal.h>
 
 #include "iperf.h"
 #include "iperf_api.h"
@@ -56,6 +57,23 @@ iperf_client_worker_run(void *s) {
     struct iperf_stream *sp = (struct iperf_stream *) s;
     struct iperf_test *test = sp->test;
 
+    /* Blocking signal to make sure that signal will be handled by main thread */
+    sigset_t set;
+    sigemptyset(&set);
+#ifdef SIGTERM
+    sigaddset(&set, SIGTERM);
+#endif
+#ifdef SIGHUP
+    sigaddset(&set, SIGHUP);
+#endif
+#ifdef SIGINT
+    sigaddset(&set, SIGINT);
+#endif
+    if (pthread_sigmask(SIG_BLOCK, &set, NULL) != 0) {
+	    i_errno = IEPTHREADSIGMASK;
+	    goto cleanup_and_fail;
+    }
+
     /* Allow this thread to be cancelled even if it's in a syscall */
     pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
     pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
@@ -130,6 +148,11 @@ iperf_create_streams(struct iperf_test *test, int sender)
 		    i_errno = IESETCONGESTION;
 		    return -1;
 		}
+	        if (test->congestion_used) {
+	          if (test->debug)
+	            printf("Overriding existing congestion algorithm: %s\n", test->congestion_used);
+	          free(test->congestion_used);
+	        }
                 // Set actual used congestion alg, or set to unknown if could not get it
                 if (rc < 0)
                     test->congestion_used = strdup("unknown");
@@ -288,6 +311,11 @@ iperf_handle_message_client(struct iperf_test *test)
 	i_errno = IEINITTEST;
         return -1;
     }
+
+    if (test->debug_level >= DEBUG_LEVEL_INFO) {
+        iperf_printf(test, "Reading new State from the Server - current state is %d-%s\n", test->state, state_to_text(test->state));
+    }
+
     /*!!! Why is this read() and not Nread()? */
     if ((rval = read(test->ctrl_sck, (char*) &test->state, sizeof(signed char))) <= 0) {
         if (rval == 0) {
@@ -299,6 +327,10 @@ iperf_handle_message_client(struct iperf_test *test)
         }
     }
 
+    if (test->debug_level >= DEBUG_LEVEL_INFO) {
+        iperf_printf(test, "State change: client received and changed State to %d-%s\n", test->state, state_to_text(test->state));
+    }
+
     switch (test->state) {
         case PARAM_EXCHANGE:
             if (iperf_exchange_parameters(test) < 0)
@@ -683,6 +715,7 @@ iperf_run_client(struct iperf_test * test)
                         i_errno = IEPTHREADCREATE;
                         goto cleanup_and_fail;
                     }
+                    sp->thread_created = 1;
                     if (test->debug_level >= DEBUG_LEVEL_INFO) {
                         iperf_printf(test, "Thread FD %d created\n", sp->socket);
                     }
@@ -721,22 +754,25 @@ iperf_run_client(struct iperf_test * test)
                     if (sp->sender) {
                         int rc;
                         sp->done = 1;
-                        rc = pthread_cancel(sp->thr);
-                        if (rc != 0 && rc != ESRCH) {
-                            i_errno = IEPTHREADCANCEL;
-                            errno = rc;
-                            iperf_err(test, "sender cancel in pthread_cancel - %s", iperf_strerror(i_errno));
-                            goto cleanup_and_fail;
-                        }
-                        rc = pthread_join(sp->thr, NULL);
-                        if (rc != 0 && rc != ESRCH) {
-                            i_errno = IEPTHREADJOIN;
-                            errno = rc;
-                            iperf_err(test, "sender cancel in pthread_join - %s", iperf_strerror(i_errno));
-                            goto cleanup_and_fail;
-                        }
-                        if (test->debug_level >= DEBUG_LEVEL_INFO) {
-                            iperf_printf(test, "Thread FD %d stopped\n", sp->socket);
+                        if (sp->thread_created == 1) {
+                            rc = pthread_cancel(sp->thr);
+                            if (rc != 0 && rc != ESRCH) {
+                                i_errno = IEPTHREADCANCEL;
+                                errno = rc;
+                                iperf_err(test, "sender cancel in pthread_cancel - %s", iperf_strerror(i_errno));
+                                goto cleanup_and_fail;
+                            }
+                            rc = pthread_join(sp->thr, NULL);
+                            if (rc != 0 && rc != ESRCH) {
+                                i_errno = IEPTHREADJOIN;
+                                errno = rc;
+                                iperf_err(test, "sender cancel in pthread_join - %s", iperf_strerror(i_errno));
+                                goto cleanup_and_fail;
+                            }
+                            if (test->debug_level >= DEBUG_LEVEL_INFO) {
+                                iperf_printf(test, "Thread FD %d stopped\n", sp->socket);
+                            }
+                            sp->thread_created = 0;
                         }
                     }
                 }
@@ -759,22 +795,25 @@ iperf_run_client(struct iperf_test * test)
         if (!sp->sender) {
             int rc;
             sp->done = 1;
-            rc = pthread_cancel(sp->thr);
-            if (rc != 0 && rc != ESRCH) {
-                i_errno = IEPTHREADCANCEL;
-                errno = rc;
-                iperf_err(test, "receiver cancel in pthread_cancel - %s", iperf_strerror(i_errno));
-                goto cleanup_and_fail;
-            }
-            rc = pthread_join(sp->thr, NULL);
-            if (rc != 0 && rc != ESRCH) {
-                i_errno = IEPTHREADJOIN;
-                errno = rc;
-                iperf_err(test, "receiver cancel in pthread_join - %s", iperf_strerror(i_errno));
-                goto cleanup_and_fail;
-            }
-            if (test->debug_level >= DEBUG_LEVEL_INFO) {
-                iperf_printf(test, "Thread FD %d stopped\n", sp->socket);
+            if (sp->thread_created == 1) {
+                rc = pthread_cancel(sp->thr);
+                if (rc != 0 && rc != ESRCH) {
+                    i_errno = IEPTHREADCANCEL;
+                    errno = rc;
+                    iperf_err(test, "receiver cancel in pthread_cancel - %s", iperf_strerror(i_errno));
+                    goto cleanup_and_fail;
+                }
+                rc = pthread_join(sp->thr, NULL);
+                if (rc != 0 && rc != ESRCH) {
+                    i_errno = IEPTHREADJOIN;
+                    errno = rc;
+                    iperf_err(test, "receiver cancel in pthread_join - %s", iperf_strerror(i_errno));
+                    goto cleanup_and_fail;
+                }
+                if (test->debug_level >= DEBUG_LEVEL_INFO) {
+                    iperf_printf(test, "Thread FD %d stopped\n", sp->socket);
+                }
+                sp->thread_created = 0;
             }
         }
     }
@@ -798,22 +837,28 @@ iperf_run_client(struct iperf_test * test)
     /* Cancel all outstanding threads */
     i_errno_save = i_errno;
     SLIST_FOREACH(sp, &test->streams, streams) {
+        if (sp->done) {
+            continue;
+        }
         sp->done = 1;
         int rc;
-        rc = pthread_cancel(sp->thr);
-        if (rc != 0 && rc != ESRCH) {
-            i_errno = IEPTHREADCANCEL;
-            errno = rc;
-            iperf_err(test, "cleanup_and_fail in pthread_cancel - %s", iperf_strerror(i_errno));
-        }
-        rc = pthread_join(sp->thr, NULL); 
-        if (rc != 0 && rc != ESRCH) {
-            i_errno = IEPTHREADJOIN;
-            errno = rc;
-            iperf_err(test, "cleanup_and_fail in pthread_join - %s", iperf_strerror(i_errno));
-        }
-        if (test->debug_level >= DEBUG_LEVEL_INFO) {
-            iperf_printf(test, "Thread FD %d stopped\n", sp->socket);
+        if (sp->thread_created == 1) {
+            rc = pthread_cancel(sp->thr);
+            if (rc != 0 && rc != ESRCH) {
+                i_errno = IEPTHREADCANCEL;
+                errno = rc;
+                iperf_err(test, "cleanup_and_fail in pthread_cancel - %s", iperf_strerror(i_errno));
+            }
+            rc = pthread_join(sp->thr, NULL); 
+            if (rc != 0 && rc != ESRCH) {
+                i_errno = IEPTHREADJOIN;
+                errno = rc;
+                iperf_err(test, "cleanup_and_fail in pthread_join - %s", iperf_strerror(i_errno));
+            }
+            if (test->debug_level >= DEBUG_LEVEL_INFO) {
+                iperf_printf(test, "Thread FD %d stopped\n", sp->socket);
+            }
+            sp->thread_created = 0;
         }
     }
     if (test->debug_level >= DEBUG_LEVEL_INFO) {
